<?php

/**
 * @file
 * Hooks and preprocess functions for the Mason module.
 */

use Drupal\Component\Serialization\Json;
use Drupal\Core\Template\Attribute;
use Drupal\blazy\Blazy;
use Drupal\mason\MasonDefault;

/**
 * Prepares variables for mason.html.twig templates.
 */
function template_preprocess_mason(&$variables) {
  $element = $variables['element'];
  foreach (MasonDefault::themeProperties() as $key) {
    $variables[$key] = isset($element["#$key"]) ? $element["#$key"] : [];
  }

  // Prepare attributes, and add the configuration as JSON object.
  $settings = &$variables['settings'];
  $settings += MasonDefault::htmlSettings();
  $id = Blazy::getHtmlId('mason', $settings['id']);

  $attributes = &$variables['attributes'];
  $attributes['id'] = $id;

  // @todo Blazy::containerAttributes($attributes, $settings); post blazy:2.x.
  $attributes['class'][] = 'blazy';
  $attributes['data-mason'] = $variables['json'];
  $attributes['data-blazy'] = empty($settings['blazy_data']) ? '' : Json::encode($settings['blazy_data']);

  if (!empty($settings['media_switch'])) {
    $switch = str_replace('_', '-', $settings['media_switch']);
    $attributes['data-' . $switch . '-gallery'] = TRUE;
  }

  // Pass settings and attributes to twig.
  $settings['class_by_id'] = strpos($id, 'mason-') !== FALSE ? str_replace('mason-', 'mason--', $id) : '';
}

/**
 * Prepares variables for mason-box.html.twig templates.
 */
function template_preprocess_mason_box(&$variables) {
  foreach (['attributes', 'content_attributes', 'delta', 'item', 'settings'] as $key) {
    $default = $key == 'delta' ? 0 : [];
    $variables[$key] = isset($variables['element']["#$key"]) ? $variables['element']["#$key"] : $default;
  }

  // Boxes may have captions.
  foreach (['alt', 'category', 'data', 'link', 'title'] as $key) {
    $variables['item']['caption'][$key] = isset($variables['item']['caption'][$key]) ? $variables['item']['caption'][$key] : [];
  }

  $item            = &$variables['item'];
  $settings        = $variables['settings'];
  $item['box']     = isset($item['box']) ? $item['box'] : [];
  $item['caption'] = array_filter($item['caption']);

  $variables['fillers'] = !empty($settings['fillers']) && ($variables['delta'] > $settings['fillers']);
  $classes = empty($variables['attributes']['class']) ? [] : $variables['attributes']['class'];
  $variables['attributes']['class'] = array_merge([($variables['fillers'] ? 'mason__fill' : 'mason__box'), 'box'], $classes);
  $content_attributes = &$variables['content_attributes'];
  $variables['content_attributes'] = new Attribute($content_attributes);
}
